package doctor.service.impl;

import doctor.common.core.domain.HealthR;
import doctor.common.security.service.TokenService;
import doctor.domain.dto.ApplyJoinDto;
import doctor.domain.entity.DoctorJobTitleEntity;
import doctor.domain.vo.DoctorVo;
import doctor.domain.vo.FindImagePicVo;
import doctor.mapper.HealthJobTitleMapper;
import doctor.service.HealthJobTitleService;
import doctor.system.api.domain.Department;
import doctor.system.api.domain.Doctor;
import doctor.system.api.model.LoginUser;
import doctor.util.OssUtil;
import doctor.util.RSAUtils;
import doctor.util.RsaKey;
import doctor.util.SendEmail;
import org.bouncycastle.jcajce.provider.asymmetric.rsa.RSAUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;
import java.util.concurrent.TimeUnit;

@Service
public class HealthJobTitleServiceImpl implements HealthJobTitleService {
    @Autowired
    private HealthJobTitleMapper healthJobTitleMapper;
    @Autowired
    private SendEmail sendEmail;
    @Autowired
    private TokenService tokenService;
    @Resource
    private RedisTemplate<String,Integer> redisTemplate;

    @Override
    public List<DoctorJobTitleEntity> findJobTitleList() {
        return healthJobTitleMapper.findJobTitleList();
    }

    @Override
    public HealthR sendEmailCode(String email) {
        if (!redisTemplate.hasKey(email)){
            Integer integer = redisTemplate.opsForValue().get(email);
            sendEmail.sendEmail(email,integer);
        }
        if (email==null){
            return HealthR.fail();
        }
        Integer code=0;
        Random random = new Random();
        for (int i = 0; i < 4; i++) {
            int i1 = random.nextInt();
            code=i+=i1;
        }
        sendEmail.sendEmail(email,code);
        redisTemplate.opsForValue().set(email,code,2, TimeUnit.MINUTES);
        return HealthR.ok();
    }

    @Override
    public HealthR applyJoin(ApplyJoinDto applyJoinDto) {
        if (applyJoinDto==null){
            return HealthR.fail("参数为空");
        }
        String s="";
        try {
             s= RSAUtils.rsaDecrypt(applyJoinDto.getPwd1(), RsaKey.PRIVATE_KEY);
            String s2 = RSAUtils.rsaDecrypt(applyJoinDto.getPwd2(), RsaKey.PRIVATE_KEY);
            if (!s.equals(s2)){
                return HealthR.fail("请输入相同的密码");
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        if (!redisTemplate.hasKey(applyJoinDto.getEmail())){
            return HealthR.fail("验证码过去请重新发送");
        }
        Integer integer = redisTemplate.opsForValue().get(applyJoinDto.getEmail());
        if (!String.valueOf(integer).equals(applyJoinDto.getCode())){
            return HealthR.fail("验车码错误");
        }
        Doctor doctor = new Doctor();
        BeanUtils.copyProperties(applyJoinDto,doctor);
        doctor.setCreateTime(new Date());
        Integer jobTitleId = applyJoinDto.getJobTitleId();
        DoctorJobTitleEntity doctorJobTitleEntity=healthJobTitleMapper.findDoctorJobTitleById(jobTitleId);
        doctor.setJobTitle(doctorJobTitleEntity.getJobTitle());
        healthJobTitleMapper.insertDoctor(doctor);
        redisTemplate.delete(applyJoinDto.getEmail());
        return HealthR.ok();
    }

    @Override
    public DoctorVo findDoctorById(Integer doctorId) {
        DoctorVo doctorVo = new DoctorVo();
        Doctor doctor=healthJobTitleMapper.findDoctorById(doctorId);
        Department department=healthJobTitleMapper.findDepartmentById(doctor.getDepartmentId());
        BeanUtils.copyProperties(doctor,doctorVo);
        BeanUtils.copyProperties(department,doctorVo);
        return doctorVo;
    }

    @Override
    public List<FindImagePicVo> findSystemImagePic() {
        ArrayList<FindImagePicVo> findImagePicVos = new ArrayList<>();
        FindImagePicVo findImagePicVo = new FindImagePicVo();
        List<String> imgPics= healthJobTitleMapper.findDoctorImgPic();
        for (String imgPic : imgPics) {
            findImagePicVo.setImagePic(imgPic);
            findImagePicVos.add(findImagePicVo);
        }
        return findImagePicVos;
    }

    @Override
    public HealthR uploadImagePic(MultipartFile imagePic,Integer doctorId,String sessionId) {
        String s = OssUtil.uploadMultipartFile(imagePic);
        healthJobTitleMapper.updDoctorImgPicBuyId(doctorId,s);
        LoginUser loginUser = tokenService.getLoginUser(sessionId);
        Doctor doctor = loginUser.getDoctor();
        doctor.setImagePic(s);
        tokenService.createToken(loginUser);
        return HealthR.ok(s);
    }
}
